@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 2001, William Chia-Wei Cheng.
+ * Copyright (C) 2001-2003, William Chia-Wei Cheng.
  *
  * This file may be distributed under the terms of the Q Public License
  * as defined by Trolltech AS of Norway and appearing in the file
@@ -15,7 +15,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
  * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/navigate.c,v 4.37 2001/09/04 14:36:37 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/navigate.c,v 4.45 2003/02/25 01:17:47 william Exp $
  */
 
 #define _INCLUDE_FROM_NAVIGATE_C_
@@ -89,6 +89,12 @@ static struct URLCacheRec *firstURLCache=NULL, *lastURLCache=NULL;
 static int maxURLCache=(-1), curURLCache=0;
 
 static int lineWidthIndexInSlideShow=4;
+static int makeUnsavableInSlideShow=FALSE;
+static int ignoreSlideShowOffsetsInFile=TRUE;
+
+static int savedSlideShowXOffset=0;
+static int savedSlideShowYOffset=0;
+static int savedSlideShowOffsets=FALSE;
 
 int ParseXYSpec(spec, pnx, pny)
    char *spec;
@@ -277,6 +283,10 @@ void CleanUpNavigate()
 
    CleanUpURLCache();
    CleanUpSlideShowInfo();
+
+   savedSlideShowXOffset = 0;
+   savedSlideShowYOffset = 0;
+   savedSlideShowOffsets = FALSE;
 }
 
 void InitNavigate()
@@ -311,6 +321,10 @@ void InitNavigate()
       UtilTrimBlanks(spec);
       if (ParseXYSpec(spec, &slideShowXOffset, &slideShowYOffset)) {
          slideShowInfoValid = TRUE;
+
+         savedSlideShowXOffset = slideShowXOffset;
+         savedSlideShowYOffset = slideShowYOffset;
+         savedSlideShowOffsets = TRUE;
       } else {
          fprintf(stderr, TgLoadString(STID_INVALID_XDEF_USE_ALT_STR),
                TOOL_NAME, "SlideShowWindowOffsets", spec, "0,0");
@@ -335,6 +349,17 @@ void InitNavigate()
       if (slideShowBorderColor == NULL) FailAllocMessage();
       slideShowInfoValid = TRUE;
    }
+   makeUnsavableInSlideShow = FALSE;
+   if ((c_ptr=XGetDefault(mainDisplay,TOOL_NAME,"MakeUnsavableInSlideShow")) !=
+         NULL && UtilStrICmp(c_ptr, "true") == 0) {
+      makeUnsavableInSlideShow = TRUE;
+   }
+   ignoreSlideShowOffsetsInFile = TRUE;
+   if ((c_ptr=XGetDefault(mainDisplay, TOOL_NAME,
+         "IgnoreSlideShowOffsetsInFile")) != NULL &&
+         UtilStrICmp(c_ptr, "false") == 0) {
+      ignoreSlideShowOffsetsInFile = FALSE;
+   }
 }
 
 static
@@ -1288,6 +1313,8 @@ void StartSlideShow()
    ClearObjCachesInAllPages();
    HidePopupMenusForSlideShow();
 
+   RecordWBSlideShow(TRUE);
+
    if (exec_attr != NULL) {
       DoExecLoop(NULL, exec_attr);
    }
@@ -1298,6 +1325,8 @@ void EndSlideShow()
 {
    XSetWindowBackground(mainDisplay, mainWindow, myBgPixel);
 
+   RecordWBSlideShow(FALSE);
+
    ShowPopupMenusForSlideShow();
    ClearObjCachesInAllPages();
    if (gstSlideShow.saved_in_hyperspace != inHyperSpace) {
@@ -1321,6 +1350,7 @@ void EndSlideShow()
    MakeQuiescent();
 }
 
+static
 void ToggleSlideShow()
 {
    int need_to_restore_cur_choice=FALSE;
@@ -1329,7 +1359,7 @@ void ToggleSlideShow()
       if (pageLayoutMode == PAGE_TILE) {
          switch (MsgBox(TgLoadString(STID_CANT_SLIDESHOW_IN_TILED_PAGE),
                TOOL_NAME, YNC_MB)) {
-         case MB_ID_YES: break;
+         case MB_ID_YES: SetUnSavableFile(TRUE); break;
          case MB_ID_NO: return;
          case MB_ID_CANCEL: return;
          }
@@ -1346,18 +1376,22 @@ void ToggleSlideShow()
             case MB_ID_CANCEL: return;
             }
          }
-         if (!IsFiletUnSavable() && NeedToFixPageNumbersForSlideShow()) {
-            sprintf(gszMsgBox,
-                  TgLoadString(STID_Q_PAGE_NUM_MOD_IN_SLIDE_YNC), TOOL_NAME);
-            switch (MsgBox(gszMsgBox, TOOL_NAME, YNC_MB)) {
-            case MB_ID_YES: SetUnSavableFile(TRUE); break;
-            case MB_ID_NO: return;
-            case MB_ID_CANCEL: return;
+         if (!IsFiletUnSavable() && (makeUnsavableInSlideShow ||
+               NeedToFixPageNumbersForSlideShow())) {
+            if (!makeUnsavableInSlideShow) {
+               sprintf(gszMsgBox,
+                     TgLoadString(STID_Q_PAGE_NUM_MOD_IN_SLIDE_YNC), TOOL_NAME);
+               switch (MsgBox(gszMsgBox, TOOL_NAME, YNC_MB)) {
+               case MB_ID_YES: break;
+               case MB_ID_NO: return;
+               case MB_ID_CANCEL: return;
+               }
             }
+            SetUnSavableFile(TRUE);
          }
-         MakeQuiescent();
-         need_to_restore_cur_choice = TRUE;
       }
+      MakeQuiescent();
+      need_to_restore_cur_choice = TRUE;
    }
    inSlideShow = !inSlideShow;
 
@@ -1373,6 +1407,16 @@ void ToggleSlideShow()
    if (need_to_restore_cur_choice) SetCurChoice(curChoiceBeforeMakeQuiescent);
 }
 
+void EnterSlideShow()
+{
+   if (!inSlideShow) ToggleSlideShow();
+}
+
+void LeaveSlideShow()
+{
+   if (inSlideShow) ToggleSlideShow();
+}
+
 void SetSlideShowBorderColor()
 {
    char spec[MAXSTRING];
@@ -1423,6 +1467,10 @@ void SetSlideShowWindowOffsets()
       sprintf(gszMsgBox, TgLoadString(STID_SLIDE_WIN_XY_OFFSETS_SET_TO),
             slideShowXOffset, slideShowYOffset);
       Msg(gszMsgBox);
+
+      savedSlideShowXOffset = slideShowXOffset;
+      savedSlideShowYOffset = slideShowYOffset;
+      savedSlideShowOffsets = TRUE;
    }
    slideShowInfoValid = TRUE;
 }
@@ -1576,9 +1624,13 @@ int ReadSlideShowInfo(buf)
          slideShowBorderColor = UtilStrDup(color_str);
          if (slideShowBorderColor == NULL) FailAllocMessage();
       }
-      slideShowXOffset = x;
-      slideShowYOffset = y;
-
+      if (!ignoreSlideShowOffsetsInFile) {
+         slideShowXOffset = x;
+         slideShowYOffset = y;
+      } else if (savedSlideShowOffsets) {
+         slideShowXOffset = savedSlideShowXOffset;
+         slideShowYOffset = savedSlideShowYOffset;
+      }
       slideShowInfoValid = TRUE;
    }
    return TRUE;
