@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 2001-2001, William Chia-Wei Cheng.
+ * Copyright (C) 2001-2003, William Chia-Wei Cheng.
  *
  * This file may be distributed under the terms of the Q Public License
  * as defined by Trolltech AS of Norway and appearing in the file
@@ -15,7 +15,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
  * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/ruler.c,v 4.22 2002/03/08 01:01:34 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/ruler.c,v 4.25 2003/02/25 01:17:48 william Exp $
  */
 
 #define _INCLUDE_FROM_RULER_C_
@@ -289,6 +289,33 @@ void PixelToMeasurementUnit(Buf, NumPixels)
    }
 }
 
+void SquarePixelToMeasurementUnit(Buf, NumSquarePixels)
+   char *Buf;
+   int NumSquarePixels;
+{
+   float fval;
+   int ival;
+
+   if (*unitStr == '\0') {
+      sprintf(Buf, "%+1d", NumSquarePixels);
+   } else {
+      fval = (float)((float)NumSquarePixels*1000.0)/(gfPixelsPerUnit);
+      fval = (float)(fval/gfPixelsPerUnit);
+      ival = ((int)round(fval));
+      fval = ((float)ival)/((float)1000.0);
+      if (*formatUnitStr == '\0') {
+         sprintf(Buf, "%+.3f sq %s", fval, unitStr);
+      } else {
+         float frac;
+
+         ival = (int)fval;
+         frac = ((ival >= 0) ? fval-((float)ival) : ((float)ival)-fval);
+         fval = (float)ival;
+         sprintf(Buf, formatUnitStr, fval, frac*gfNumFracUnits);
+      }
+   }
+}
+
 void RedrawHRuler(dpy, win)
    Display *dpy;
    Window win;
