@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 2001-2001, William Chia-Wei Cheng.
+ * Copyright (C) 2001-2003, William Chia-Wei Cheng.
  *
  * This file may be distributed under the terms of the Q Public License
  * as defined by Trolltech AS of Norway and appearing in the file
@@ -15,7 +15,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
  * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/select.c,v 4.40 2002/01/05 02:48:06 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/select.c,v 4.43 2003/02/25 01:17:48 william Exp $
  */
 
 #define _INCLUDE_FROM_SELECT_C_
@@ -1526,7 +1526,7 @@ void KeyPressInSelect(key_ev)
    TranslateKeys(s, &key_sym);
    if (inSlideShow) {
       if (CharIsESC(key_ev, s, key_sym, &has_ch)) {
-         ToggleSlideShow();
+         LeaveSlideShow();
          return;
       }
       switch (key_sym) {
@@ -2210,8 +2210,8 @@ void Teleport(button_ev)
    DoExecLoop(obj_ptr, exec_attr);
 }
 
-void SelAllObj(high_light)
-   int high_light;
+void SelAllObj(high_light, ignore_slideshow)
+   int high_light, ignore_slideshow;
 {
    struct ObjRec *obj_ptr=NULL;
 
@@ -2221,7 +2221,7 @@ void SelAllObj(high_light)
       HighLightReverse();
       RemoveAllSel();
    }
-   if (inSlideShow) {
+   if (inSlideShow && !ignore_slideshow) {
       SetCurChoice(curChoiceBeforeMakeQuiescent);
       return;
    }
