@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 2001, William Chia-Wei Cheng.
+ * Copyright (C) 2001-2003, William Chia-Wei Cheng.
  *
  * This file may be distributed under the terms of the Q Public License
  * as defined by Trolltech AS of Norway and appearing in the file
@@ -15,7 +15,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
  * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/shape.c,v 4.16 2001/08/06 21:39:13 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/shape.c,v 4.20 2003/02/25 01:17:48 william Exp $
  */
 
 #define _INCLUDE_FROM_SHAPE_C_
@@ -818,7 +818,7 @@ void GenerateShadow()
    struct ObjRec *obj_ptr;
    struct SelRec *tmp_top_sel=NULL, *tmp_bot_sel=NULL;
 
-   SelAllObj(FALSE);
+   SelAllObj(FALSE, FALSE);
 
    JustDupSelObj(&tmp_top_sel, &tmp_bot_sel);
    /*
@@ -957,7 +957,7 @@ struct AttrRec *GenerateShape()
     * Use the next two functions to set selLtX, etc. to be used
     *    in CreateGroupObj().
     */
-   SelAllObj(FALSE);
+   SelAllObj(FALSE, FALSE);
    RemoveAllSel();
 
    tmp_top_obj = topObj;
@@ -1042,7 +1042,7 @@ void ContinueCreateShape(orig_x, orig_y)
          }
          DoCursor(grid_x, grid_y, grid_x-orig_x, grid_y-orig_y, MOTION_SHOW);
          HighLightCreateShape(grid_x, grid_y, GENERATE, &need_to_draw);
-         DrawIntervalRulers(orig_x, orig_y, grid_x, grid_y);
+         DrawIntervalRulers(orig_x, orig_y, grid_x, grid_y, NULL);
          while (XCheckMaskEvent(mainDisplay, PointerMotionMask, &ev)) ;
          break;
       case KeyPress:
