@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 2001-2001, William Chia-Wei Cheng.
+ * Copyright (C) 2001-2003, William Chia-Wei Cheng.
  *
  * This file may be distributed under the terms of the Q Public License
  * as defined by Trolltech AS of Norway and appearing in the file
@@ -15,7 +15,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
  * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/shortcut.c,v 4.83 2002/01/05 02:48:06 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/shortcut.c,v 4.93 2003/08/16 19:38:53 william Exp $
  */
 
 #define _INCLUDE_FROM_SHORTCUT_C_
@@ -413,6 +413,15 @@ static struct ShortCutRec shortCutXlateTbl[] = {
    { '\0', 0, "AddPlusTickMarks()", 0, CMDID_ADDPLUSTICKMARKS },
    { '\0', 0, "AddHourGlassTickMarks()", 0, CMDID_ADDHOURGLASSTICKMARKS },
    { '\0', 0, "SetTickMarkSize()", 1, CMDID_SETTICKMARKSIZE },
+   { '\0', 0, "ToggleShowChat()", 0, CMDID_TOGGLESHOWCHAT },
+   { '\0', 0, "SavePagesAs()", 0, CMDID_SAVEPAGESAS },
+   { '\0', 0, "ImportPPMFile()", 0, CMDID_IMPORTPPMFILE },
+   { '\0', 0, "AddPageBeforeCopyAll()", 0, CMDID_ADDPAGEBEFORECOPYALL },
+   { '\0', 0, "AddPageAfterCopyAll()", 0, CMDID_ADDPAGEAFTERCOPYALL },
+   { '\0', 0, "InsertHexOctalChar()", 0, CMDID_INSERTHEXOCTALCHAR },
+   { '\0', 0, "ResetInputMethod()", 0, CMDID_RESETINMETHOD },
+   { '\0', 0, "ImportPBMFile()", 0, CMDID_IMPORTPBMFILE },
+   { '\0', 0, "ImportPGMFile()", 0, CMDID_IMPORTPGMFILE },
    { '\0', 0, "", 0, 0 }
 };
 
